# -*- encoding:utf-8 -*-
import json
import re


def inner_post_processing(base_info):
    print(json.dumps(base_info,ensure_ascii=False,indent=4))
    """
    处理 '基础信息' 部分，提取所需字段。

    参数:
        base_info (dict): 包含 '基础信息' 的字典。

    返回:
        dict: 提取的信息字典 extracted_info。
    """
    # 初始化提取的信息字典
    extracted_info = {}

    # 定义一个辅助函数用于获取嵌套字典中的值
    def get_nested(dic, keys, default=None):
        for key in keys:
            if isinstance(dic, dict):
                dic = dic.get(key, default)
            else:
                return default
        return dic

    # 定义一个辅助函数用于递归查找包含特定子字符串的键
    def find_keys_containing(dic, substring):
        found_values = []
        if isinstance(dic, dict):
            for key, value in dic.items():
                if substring in key:
                    found_values.append(value)
                if isinstance(value, dict):
                    found_values.extend(find_keys_containing(value, substring))
                elif isinstance(value, list):
                    for item in value:
                        if isinstance(item, dict):
                            found_values.extend(find_keys_containing(item, substring))
        return found_values

    # 定义一个辅助函数用于根据候选键列表提取值（部分匹配）
    def extract_field(contact_info, candidate_keys):
        for candidate in candidate_keys:
            for key, value in contact_info.items():
                if candidate in key and value not in ["未知", ""]:
                    return value
        return ""

    # 定义一个辅助函数用于提取 '投标保证金'
    def extract_bid_bond(guarantee_info):
        # 定义投标保证金的候选键
        bid_bond_candidates = ["投标保证金", "磋商保证金"]

        # 第一步：查找包含 "投标保证金" 或 "磋商保证金" 的键
        for candidate in bid_bond_candidates:
            for key, value in guarantee_info.items():
                if candidate in key:
                    if isinstance(value, dict):
                        # 在嵌套字典中查找包含 "金额" 的键
                        for sub_key, sub_value in value.items():
                            if "金额" in sub_key and sub_value not in ["未知", ""]:
                                return sub_value
                    elif isinstance(value, str):
                        if "金额" in key and value not in ["未知", ""]:
                            return value
                    else:
                        # 如果 value 既不是 dict 也不是 str，忽略
                        continue

        # 第二步：如果没有找到包含 "金额" 的键，尝试在所有键值中查找符合模式的值
        amount_pattern = re.compile(r'(?:\d{1,3}(?:[,，]\d{3})*(?:\.\d+)?|\d+(?:\.\d+)?|[\u4e00-\u9fff]+(?:\.\d+)?)\s*(?:元|万元)')
        for key, value in guarantee_info.items():
            if isinstance(value, str):
                match = amount_pattern.search(value)
                if match:
                    return match.group()
            elif isinstance(value, dict):
                # 递归查找嵌套字典中的金额
                found_amount = extract_bid_bond(value)
                if found_amount:
                    return found_amount
        # 如果都没有找到，则返回空字符串
        return ""

    # 定义所需字段的映射关系，暂时不包含'联系人'和'联系电话'以及'招标项目地点'
    mapping = {
        "代理机构名称": [["招标人/代理信息", "招标代理机构"]],
        "招标项目名称": [["项目信息", "项目名称"], ["项目信息", "工程名称"]],
        "招标项目编号": [["项目信息", "项目编号"], ["项目信息", "招标编号"]],
        "开标时间": [["关键时间/内容", "开标时间"]],
        "报名截止日期": [["关键时间/内容", "投标文件递交截止日期"]],
        "招标项目预算": [["项目信息", "招标控制价"]],
        "招标单位名称": [["招标人/代理信息", "招标人"]],
        "招标公告地址": [["关键时间/内容", "信息公示媒介"], ["关键时间/内容", "评标结果公示媒介"]],
    }

    # 提取并映射字段
    for new_key, paths in mapping.items():
        value = None
        for path in paths:
            value = get_nested(base_info, path)
            if value:
                break
        extracted_info[new_key] = value if value else ""

    # 特殊处理 '招标项目地点'
    # 在 '项目信息' 下查找包含 "地点" 的键
    project_info = base_info.get("项目信息", {})
    location_candidates = find_keys_containing(project_info, "地点")
    if location_candidates:
        # 选择第一个找到的地点
        extracted_info["招标项目地点"] = location_candidates[0]
    else:
        extracted_info["招标项目地点"] = ""

    # 特殊处理 '联系人' 和 '联系电话'
    # 提取 '项目联系方式'
    project_contact = get_nested(base_info, ["招标人/代理信息", "项目联系方式"], {})
    if not isinstance(project_contact, dict):
        project_contact = {}

    # 提取 '招标人联系方式'
    bidder_contact = get_nested(base_info, ["招标人/代理信息", "招标人联系方式"], {})
    if not isinstance(bidder_contact, dict):
        bidder_contact = {}

    # 定义候选键列表，按优先级排序
    name_candidates = ["联系人","名称", "招标"]
    phone_candidates = ["电话", "手机", "联系方式"]

    # 提取 '联系人'
    contact_names = [project_contact, bidder_contact]
    contact_name = ""
    for contact in contact_names:
        extracted_name = extract_field(contact, name_candidates)
        if extracted_name:
            contact_name = extracted_name
            break
    extracted_info["联系人"] = contact_name

    # 提取 '联系电话'
    contact_phones = [project_contact, bidder_contact]
    contact_phone = ""
    for contact in contact_phones:
        extracted_phone = extract_field(contact, phone_candidates)
        if extracted_phone:
            contact_phone = extracted_phone
            break
    extracted_info["联系电话"] = contact_phone

    # 特殊处理 '投标保证金'
    # 提取 '保证金相关'
    guarantee_info = get_nested(base_info, ["保证金相关"], {})
    extracted_info["投标保证金"] = extract_bid_bond(guarantee_info)

    return extracted_info

def outer_post_processing(combined_data, includes):
    """
    外层处理函数，调用内层 post_processing 处理 '基础信息'，并构建 processed_data。
    额外提取 '采购要求' 下的 '技术要求' 内容。

    参数:
        combined_data (dict): 原始合并数据。
        includes (list): 需要包含的键列表。

    返回:
        tuple: (processed_data, extracted_info, procurement_reqs)
    """
    # 初始化结果字典，预设'其他'分类为空字典
    processed_data = {"其他": {}}

    # 初始化提取的信息字典
    extracted_info = {}

    # 初始化采购要求的技术要求
    procurement_reqs = ""

    # 检查 '基础信息' 是否在 includes 中
    if "基础信息" in includes:
        base_info = combined_data.get("基础信息", {})
        # 调用内层 post_processing 处理 '基础信息'
        extracted_info = inner_post_processing(base_info)
        # 将 '基础信息' 保留在处理后的数据中
        processed_data["基础信息"] = base_info

        # 提取 '采购要求' 下的 '技术要求'
        procurement_reqs = base_info.get("采购要求", {}).get("技术要求", "")
        if not procurement_reqs:
            # 如果 '技术要求' 不存在或为空，可以根据需要设置默认值
            procurement_reqs = "未提供"  # 或者其他适当的默认值

    # 遍历原始字典的每一个键值对
    for key, value in combined_data.items():
        if key in includes:
            if key == "基础信息":
                # 已经处理 '基础信息'，无需再次添加
                continue
            else:
                # 直接保留包含在 includes 列表中的键值对
                processed_data[key] = value
        else:
            # 将不在 includes 列表中的键值对加入到 '其他' 分类中
            processed_data["其他"][key] = value

    # 如果 '其他' 分类没有任何内容，可以选择删除这个键
    if not processed_data["其他"]:
        del processed_data["其他"]

    return processed_data, extracted_info, procurement_reqs

if __name__ == "__main__":
    combined_data={
    "投标文件要求": {
        "投标文件": {
            "投标文件的组成": [
                "投标文件应包括下列内容：（1）投标函及投标函附录；（2）法定代表人身份证明；（3）联合体协议书；（4）投标保证金；（5）已标价工程量清单；（6）施工组织设计；（7）项目管理机构；（8）拟分包项目情况表；（9）资格审查资料；（10）投标人须知前附表规定的其他材料。",
                "投标人须知前附表规定不接受联合体投标的，或投标人没有组成联合体的，投标文件不包括本章第3.1.1（3）目所指的联合体协议书。"
            ],
            "投标报价": [
                "投标人应按第五章“工程量清单”的要求填写相应表格，并按本章第2.1.2（1）目约定的计税方法计税。未按约定的计税方法计税的，其投标将被否决。",
                "投标人经复核认为招标人公布的招标控制价未按本章第2.1.2（1）目、第2.1.2（3）目的规定编制，可按本章第2.4款的规定向招标人提出异议。对异议的答复仍持有异议，可按本章第9.5款的规定在投标截止时间前，向有关行政监督部门和工程造价管理机构投诉。",
                "投标人在投标截止时间前修改投标函中的投标总报价，应同时修改第五章“工程量清单”中的相应报价。此修改须符合本章第4.3款的有关要求。",
                "投标人投标函中的大写报价或算术错误修正后的投标总报价大于招标控制价总价的，其投标将被否决。"
            ],
            "投标有效期": [
                "在投标人须知前附表规定的投标有效期内，投标人不得要求撤销或修改其投标文件。",
                "出现特殊情况需要延长投标有效期的，招标人以书面形式通知所有投标人延长投标有效期。投标人同意延长的，应相应延长其投标保证金的有效期，但不得要求或被允许修改或撤销其投标文件；投标人拒绝延长的，其投标失效，但投标人有权收回其投标保证金。"
            ],
            "投标保证金": [
                "投标人须知前附表规定提交投标保证金的，投标人在递交投标文件的同时，应按投标人须知前附表规定的形式、金额、递交截止时间、递交方式提交投标保证金，并作为其投标文件的组成部分。联合体投标的，其投标保证金由牵头人递交，并应符合投标人须知前附表的规定。",
                "投标人不按本章第3.4.1项要求提交投标保证金的，其投标将被否决。",
                "采用现金方式缴纳的，招标人最迟在与中标人签订合同后5日内，向未中标的投标人和中标人退还投标保证金。招标人同时退还投标保证金的银行同期存款利息。投标保证金及利息的计息标准和退还方式见投标人须知前附表。采用保函、保证保险和其他方式缴纳的，按照保函、保险等相关规定执行。",
                "有下列情形之一的，投标保证金将不予退还：（1）投标人在规定的投标有效期内撤销或修改其投标文件；（2）中标人在收到中标通知书后，无正当理由拒签合同、在签订合同时向招标人提出附加条件或未按招标文件规定提交履约保证金。"
            ],
            "资格审查资料（适用于未进行资格预审的）": [
                "投标人应按招标文件第八章“投标文件格式”中规定的表格内容填写资格审查表，并按各资格审查表的具体要求提供相关证件及证明材料。本招标文件中“类似项目”的定义见投标人须知前附表。",
                "投标人须知前附表第1.4.2项规定接受联合体投标的，本章第3.5.1项规定的表格和资料应包括联合体各方相关情况。"
            ],
            "备选投标方案": "除投标人须知前附表另有规定外，投标人不得递交备选投标方案。允许投标人递交备选投标方案的，只有中标人所递交的备选投标方案方可予以考虑。评标委员会认为中标候选人的备选投标方案优于其按照招标文件要求编制的投标方案的，该中标候选人被确定中标人后，招标人可以接受该备选投标方案。",
            "投标文件的编制": [
                "投标文件应按第八章“投标文件格式”进行编写，如有必要，可以增加附页，作为投标文件的组成部分。其中，投标函附录在满足招标文件实质性要求的基础上，可以提出比招标文件要求更有利于招标人的承诺。",
                "投标文件应当对招标文件有关工期、投标有效期、质量要求、技术标准和要求、招标范围等实质性内容作出响应。",
                "电子投标文件制作（1）电子投标文件由投标人使用“电子交易系统”提供的“电子投标文件制作软件”制作生成。（2）电子投标文件中的已标价工程量清单数据文件应符合《湖北省建设工程造价应用软件数据交换规范》和《关于增加湖北省建设工程造价数据接口内容的通知》（鄂建标函[2018]6号）的要求。（3）第八章投标文件格式文件要求“盖单位章”的地方，投标人应使用CA数字证书加盖投标人的单位电子印章；要求“签字”的地方，投标人应使用CA数字证书加盖法定代表人的个人电子印章或电子签名章。联合体投标的，投标文件由联合体牵头人按上述规定在要求“盖单位章”的地方加盖联合体牵头人单位电子印章；在要求“签字”的地方加盖联合体牵头人法定代表人的个人电子印章或电子签名章。招标文件有特别说明的除外。（4）电子投标文件制作完成后，将生成一份加密的电子投标文件和一份不加密的电子投标文件。（5）电子投标文件制作的具体方法详见“电子投标文件制作软件”中的帮助文档。",
                "投标人须知前附表规定“施工组织设计”采用无标识“技术暗标”时，则“施工组织设计”按本章第3.7.3项制作电子投标文件时应满足下列要求：（1）封面设置要求：采用A4规格白色底色，写明“（项目名称）（标段名称）施工组织设计”字样，文字为黑色二号宋体，可加粗；（2）目录、正文标题（包括章、节、条、款、项）、正文要求：采用A4规格白色底色，文字为黑色小四号宋体，标题可加粗；（3）图表要求：图表应尽可能采用A4规格白色底色，对于比较大的图表可使用A3规格白色底色。图表中的文字采用黑色，字体、字号不限；（4）页眉和页脚（包括页码）设置要求：不允许出现页眉，且页脚只准出现页码，页码格式采用阿拉伯数字格式，字体为五号宋体，设在页脚居中位置，页码应当连续；（5）任何情况下，施工组织设计中不得出现投标人的名称和其它可识别投标人身份的字符、徽标、人员名称等。"
            ]
        },
        "投标": {
            "投标文件的加密": "投标人应当按照本章第3.7.3项要求制作电子投标文件，并在投标时上传加密的电子投标文件，未加密的电子投标文件，招标人（“电子交易系统”）将拒收并提示。",
            "投标文件的递交": [
                "投标人递交投标文件的截止时间（投标截止时间）：见投标人须知前附表。",
                "投标人应当在投标截止时间前，通过互联网使用CA数字证书登录“电子交易系统”，选择所投标段将加密的电子投标文件上传。投标人完成投标文件上传后，“电子交易系统”即时向投标人发出电子签收凭证，递交时间以电子签收凭证载明的传输完成时间为准。投标人应充分考虑上传文件时的不可预见因素，投标文件未在投标截止时间前完成上传的，视为逾期送达，招标人（“电子交易系统”）将拒收。",
                "除投标人须知前附表另有规定外，投标人所递交的投标文件不予退还。"
            ],
            "投标文件的修改与撤回": [
                "在本章第4.2.1项规定的投标截止时间前，投标人可以修改或撤回已递交的投标文件。",
                "投标人撤回投标文件的，在“电子交易系统”直接进行撤回操作。",
                "投标人修改投标文件的，应当先按本章第4.3.2项的规定撤回投标文件，再使用“电子投标文件制作软件”制作成完整的投标文件，并按照本章第3条、第4条规定进行编制和递交。"
            ]
        }
    },
    "开评定标流程": {
        "开标": {
            "开标时间和地点": [
                "招标人在本章第4.2.1项规定的投标截止时间（开标时间）在“电子交易系统”上公开进行开标，所有投标人均应当准时在线参加开标。",
                "招标人通过互联网在投标人须知前附表规定的地点组织开标，并在投标截止时间在线开标的准备工作。30分钟前，使用CA数字证书登录“电子交易系统”，进入“开标大厅”选择相应标段作准备。",
                "投标人应当在能够保证设施设备可靠、互联网畅通的任意地点，通过互联网在线参加开标。在投标截止时间前，使用加密其投标文件的CA数字证书（企业锁）登录“电子交易系统”，进入“开标大厅”选择所投标段进行签到，并实时在线关注招标人的操作情况。"
            ],
            "开标程序": [
                "主持人按下列程序在“电子交易系统”的“开标大厅”进行在线开标：（1）宣布开标纪律；（2）公布主持人、招标人代表、监标人等有关人员姓名；（3）公布在投标截止时间前投标文件的递交情况；（4）公布投标保证金递交情况；（5）投标人根据提示在投标人须知前附表规定的时间内解密投标文件；（6）读取已解密的投标文件的内容；（7）公布投标人名称、标段名称、投标保证金的递交情况、投标报价、项目经理姓名及其他内容，并生成开标记录；（8）开标结束。",
                "在本章第5.2.1（5）目规定的时间内，非因“电子交易系统”原因造成投标文件未解密的，视为投标人撤回投标文件。在部分投标文件未解密的情况下，已解密的投标文件少于三个的，招标失败；已解密的投标文件不少于三个，开标继续进行。"
            ],
            "开标异议": [
                "投标人对开标有异议的，应当在开标过程中提出；招标人当场对异议作出答复，并记入开标记录。异议与答复应在“不见面开标大厅”在“提出异议”菜单以书面形式进行。本处所称异议是指投标人在开标过程中对投标文件提交、投标截止时间、开标程序、开标记录以及投标人和招标人或者投标人相互之间存在利益冲突的情形等提出的质疑。",
                "投标人异议成立的，招标人将及时采取纠正措施，或者提交评标委员会评审确认；投标人异议不成立的，招标人将当场给予解释说明。"
            ],
            "特殊情况的处置": [
                "因“电子交易系统”系统故障导致无法投标的，交易中心及时通知招标人，招标人视情况决定是否顺延投标截止时间。因投标人自身原因导致无法完成投标的，由投标人自行承担后果。",
                "因“电子交易系统”系统故障导致无法正常开标的，招标人将暂停开标，待系统恢复正常后继续开标。",
                "“电子交易系统”系统故障是指下列情形：（1）系统服务器发生故障，无法访问或无法使用系统；（2）系统的软件或数据库出现错误，不能进行正常操作；（3）系统发现有安全漏洞，有潜在的泄密危险；（4）出现断电、断网事故；（5）其他无法保证招投标过程正常进行的情形。",
                "因本工程采用远程不见面开标模式，故招标人特别说明如下：（1）远程开标项目的时间均以国家授时中心发布的时间为准。（2）本项目招标文件和投标文件必须使用黄石招投标工具软件编制，并通过黄石市工程建设电子交易系统完成投标过程。投标人的投标文件的编制和递交，应依照招标文件的规定进行。如未按招标文件要求编制、递交电子投标文件，将可能导致无效投标，其后果由投标人自负。（3）投标人通过黄石市工程建设电子交易系统递交的电子投标文件为评标依据，投标人使用黄石投标编制工具制作电子投标文件时生成两份文件，其中已加密投标文件用于上传至网上，开标当日，投标人不必抵达开标现场，仅需在任意地点通过不见面交易系统参加开标会议，并根据需要使用不见面交易系统与招标人进行互动交流以及文件传送等操作。投标人如来开标现场参加开标会议的，招标方和交易中心不提供上网终端设备和不保证网络通畅。（4）投标文件递交截止时间前，各投标人的授权委托人或法人代表应提前进入不见面交易系统进行在线签到，并播放远程开标会议温馨提示测试音频，收听、观看实时音视频交互效果并及时在群聊板中反馈。（5）未在投标文件递交截止时间之前进行在线签到或未能在开标会议区内全程参与交互的，视为放弃交互和放弃对开评标全过程提疑的权利，投标人将无法进行解密、唱标、评审结果查看等操作，并承担由此导致的一切后果。（6）投标文件递交截止时间后，主持人将在系统内公布投标人名单并展示投标保证金递交匹配情况（如有），然后通过开标会议区发出投标文件解密的指令，投标人在各自地点按规定时间自行实施远程解密，投标人解密需在招标代理设置的规定时间之内完成。①因投标人网络与电源不稳定、未按操作手册要求配置软硬件、解密锁发生故障或使用不当（用错CA锁、CA锁已过期、CA锁加解密功能有问题等等）、故意不在要求时限内完成解密等自身原因，导致投标文件在规定时间内未能解密、解密失败或解密超时，视为投标人放弃投标；（投标文件加解密需使用企业锁）②因招标人原因或网上招投标平台发生故障等，导致无法按时完成投标文件解密或开、评标工作无法进行的，可根据实际情况相应延迟解密时间或调整开、评标时间。③因不可抗力因素导致无法按时完成投标文件解密或开、评标工作无法进行的，可根据实际情况相应延迟解密时间或调整开、评标时间（7）若投标人已申请多把CA锁，请注意使用差别，确保制作上传加密投标文件和开标解密时使用的是企业锁CA锁（投标文件加解密需使用企业锁），否则造成解密失败的，由投标人负责。（8）开评标全过程中，各投标人参与远程交互的授权委托人或法人代表应始终为同一个人，中途不得更换，在解密、唱标、传送文件等特殊情况下需要交互时，投标人一端参与交互的人员将均被视为是投标人的授权委托人或法人代表，投标人不得以不承认交互人员的资格或身份等为借口抵赖推脱，投标人自行承担随意更换人员所导致的一切后果。（9）为顺利实现本项目开评标的远程交互，建议投标人配置的硬件设施有：高配置电脑、高速稳定的网络、电源（不间断）、CA锁、音视频设备（耳麦、话筒、高清摄像头、音响）等；建议投标人具备的软件设施有：IE浏览器（版本必须为11及以上），安装品茗驱动插件。为保证交互效果，建议投标人选择封闭安静的地点参与远程交互。因投标人自身软硬件配备不齐全或发生故障等问题而导致在交互过程中出现不稳定或中断等情况的，由投标人自身承担一切后果。（10）投标单位应充分考虑到网络及系统平台可能存在的非正常情况，在投标文件编制完成后尽早完成上传。",
                "电子投标文件雷同认定与处理（—）电子开评标系统提示不同投标人的电子投标文件的文件制作机器码，如有相同的，且投标人不能合理说明或者不能提供相应证明材料的，评标委员会应当集体认定为《中华人民共和国招标投标法实施条例》第四十条第(一)项“不同投标人的投标文件由同一单位或者个人编制”的情形。（二）电子开评标系统提示不同投标人的已标价工程量清单的文件制作机器码，如有相同的，且投标人不能合理说明或者不能提供相应证明材料的，评标委员会应当集体认定《中华人民共和国招标投标法实施条例》第四十条第(一)项“不同投标人的投标文件由同一单位或者个人编制”的情形。（三）电子开评标系统提示不同投标人的已标价工程量清单的造价软件锁号，如有相同的，且投标人不能合理说明或者不能提供相应证明材料的，评标委员会应当集体认定为《中华人民共和国招标投标法实施条例》第四十条第(一)项“不同投标人的投标文件由同一单位或者个人编制”的情形。（四）电子开评标系统提示不同投标人的投标技术文件存在内容高度一致的、投标报价或投标报价组成出现非正常一致或者呈规律性变化的，评标委员会可以要求投标人作出澄清，评标委员会应当集体认定是否属于《中华人民共和国招标投标法实施条例》第四十条第(四)项“不同投标人的投标文件异常一致或者投标报价呈规律性差异”的情形。"
            ]
        },
        "评标": {
            "评标委员会": [
                "评标由招标人依法组建的评标委员会负责。评标委员会由招标人或其委托的招标代理机构熟悉相关业务的代表，以及有关技术、经济等方面的专家组成。评标委员会成员人数以及技术、经济等方面专家的确定方式见投标人须知前附表。",
                "评标委员会成员有下列情形之一的，应当回避：（1）投标人或投标人的主要负责人的近亲属；（2）项目主管部门或者行政监督部门的人员；（3）与投标人有经济利益关系或其他利害关系，可能影响对投标公正评审的；（4）曾因在招标、评标以及其他与招标投标有关活动中从事违法行为而受过行政处罚或刑事处罚的。"
            ],
            "评标原则": "评标活动遵循公平、公正、科学和择优的原则。",
            "评标": "评标委员会按照第三章“评标办法”规定的方法、评审因素、标准和程序对投标文件进行评审。第三章“评标办法”没有规定的方法、评审因素和标准，不作为评标依据。",
            "评标结果（定标候选人）公示": "招标人将自收到评标报告之日起3日内，在投标人须知前附表规定的媒介公示定标候选人。公示期不少于3日。投标人或者其他利害关系人对评标结果有异议的，应当在评标结果公示期间提出。招标人自收到异议之日起3日内作出答复；作出答复前，暂停招标投标活动。异议与答复应当通过“电子交易系统”在“异议与答复”菜单以书面形式进行。",
            "履约能力的审查（如有）": "如果定标候选人的经营、财务状况发生较大变化或者存在违法行为，招标人认为可能影响其履约能力的，将在发出中标通知书前，召集原评标委员会按照招标文件规定的标准和方法审查确认。"
        },
        "定标": {
            "定标方式": "招标人依据评标委员会推荐的定标候选人确定中标人，评标委员会推荐定标候选人的人数见投标人须知前附表。",
            "中标通知": "评标结果公示期满后，在本章第3.3款规定的投标有效期内，招标人通过“电子交易系统”以书面形式向中标人发出中标通知书，同时将中标结果通知未中标的投标人。中标通知书发出的同时，招标人将在投标人须知前附表第6.4款规定的媒介发布中标结果公示。",
            "履约保证金": "中标人不能按本章第7.3.1项要求提交履约保证金的，视为放弃中标，其投标保证金不予退还，给招标人造成的损失超过投标保证金数额的，中标人还应当对超过部分予以赔偿。",
            "签订合同": "中标人应当在合同签订后15日内，登录“电子交易系统”提交电子合同文件归档。"
        },
        "中标": {
            "中标人的投标文件": "中标人在签订合同前须向招标人另行提交的纸质投标文件份数：见投标人须知前附表。"
        }
    },
    "无效标与废标项": {
        "否决和无效投标情形": [
            "投标人投标函中的大写报价或算术错误修正后的投标总报价大于招标控制价总价的，其投标将被否决。",
            "投标人应按第五章“工程量清单”的要求填写相应表格，并按本章第2.1.2（1）目约定的计税方法计税。未按约定的计税方法计税的，其投标将被否决。",
            "偏差在±1%以上的，或者修正后的投标价格超过招标控制价总价的（如有），其投标应当被否决。",
            "本项目招标文件和投标文件必须使用黄石招投标工具软件编制，并通过黄石市工程建设电子交易系统完成投标过程。投标人的投标文件的编制和递交，应依照招标文件的规定进行。如未按招标文件要求编制、递交电子投标文件，将可能导致无效投标，其后果由投标人自负。",
            "投标人有以下情形之一的，评标委员会应当否决其投标：<br>（1）不按评标委员会要求澄清、说明或补正；<br>（2）有串通投标、弄虚作假、行贿或有其他违法行为。<br>1）有下列情形之一的，视为投标人相互串通投标：<br>①不同投标人的投标文件由同一单位或者个人编制；<br>②不同投标人委托同一单位或者个人办理投标事宜；<br>③不同投标人的投标文件载明的项目管理成员为同一人；<br>④不同投标人的投标文件异常一致或者投标报价呈规律性差异；<br>⑤不同投标人的投标文件相互混装；<br>⑥不同投标人的投标保证金从同一单位或者个人的账户转出；<br>⑦不同投标人的投标文件存在“文件创建标识码”、“文件制作机器码”一致等情形。<br>2）有下列情形之一的，属于弄虚作假行为：<br>①使用通过受让或者租借等方式获取的资格、资质证书投标的，即以他人名义投标的。<br>②使用伪造、变造的许可证件；<br>③提供虚假的财务状况或者业绩；<br>④提供虚假的项目负责人或者主要技术人员简历、劳动关系证明；<br>⑤提供虚假的信用状况；<br>⑥其他弄虚作假的行为。",
            "未按规定提交投标保证金的投标文件将被视为非实质响应而予以拒绝。",
            "投标人不按本章第3.4.1项要求提交投标保证金的，其投标将被否决。",
            "投标报价有算术错误的，评标委员会按以下原则对投标报价进行修正，修正的价格经投标人书面确认后具有约束力。投标人不接受修正价格的，或者修正后的价格与投标报价相比偏差率（（修正后的价格-投标报价）÷投标报价×100%）在±1%以上的，或者修正后的投标报价超过招标控制价的（如有），评标委员会应当否决其投标。<br>（1）投标文件中的大写金额与小写金额不一致的，以大写金额为准；<br>（2）总价金额与依据单价计算出的结果不一致的，以单价金额为准修正总价，但单价金额小数点有明显错误的除外。",
            "投标人或其投标文件有下列情形之一的，评标委员会应当否决其投标：<br>B1.1在形式评审、响应性评审中，评标委员会认定投标人的投标文件不符合评标办法前附表中规定的任何一项评审标准的。<br>B1.2不按评标委员会要求澄清、说明或补正的。<br>B1.3有串通投标、弄虚作假、行贿或有其他违法行为的。<br>B1.4当投标人资格条件发生变化时，其在投标文件中更新的资料，未能通过资格评审的(适用于已进行资格预审的)。"
        ],
        "废标项": "",
        "不得存在的其他情形": [
            "为招标人不具有独立法人资格的附属机构（单位）",
            "与招标人存在利害关系且可能影响招标公正性",
            "与本标段的其他投标人为同一个单位负责人",
            "与本标段的其他投标人存在控股、管理关系",
            "为本标段前期准备提供设计或咨询服务的",
            "为本标段的监理人",
            "为本标段的代建人",
            "为本标段提供招标代理服务",
            "与本标段的监理人或代建人或招标代理机构同为一个法定代表人",
            "与本标段的监理人或代建人或招标代理机构相互控股或参股",
            "与本标段的监理人或代建人或招标代理机构相互任职或工作",
            "被依法暂停或取消投标资格（指被本招标项目所在地县级及以上住房城乡建设主管部门或其他行政主管部门暂停或取消投标资格或禁止进入该区域建设市场且处于有效期内）",
            "被责令停产停业、暂扣或者吊销许可证、暂扣或者吊销执照",
            "进入清算程序，或被宣告破产，或其他丧失履约能力的情形",
            "在最近三年内发生重大工程质量问题（以相关行业主管部门的行政处罚决定或司法机关出具的有关法律文书为准）",
            "在“国家企业信用信息公示系统”（www.gsxt.gov.cn）中被列入严重违法失信企业名单",
            "在“信用中国”网站（www.creditchina.gov.cn）被列入失信被执行人名单",
            "在近三年内投标人或其法定代表人、拟委任的项目经理有行贿犯罪行为",
            "法律法规或投标人须知前附表规定的其他情形",
            "投标人不得相互串通投标或者与招标人串通投标，不得向招标人或者评标委员会成员行贿谋取中标，不得以他人名义投标或者以其他方式弄虚作假骗取中标；投标人不得以任何方式干扰、影响评标工作"
        ]
    },
    "基础信息": {
        "招标人/代理信息": {
            "招标人": "见投标人须知前附表",
            "招标人联系方式": "未知",
            "招标代理机构": "见投标人须知前附表",
            "招标代理机构联系方式": "未知"
        },
        "项目信息": {
            "项目名称": "大冶市 2021年老旧小区改造工程 （东岳街道）二标段",
            "招标编号": "G2022012801000",
            "项目概况": {
                "项目名称": "见投标人须知前附表",
                "建设地点": "见投标人须知前附表",
                "设计人": "见申请人须知前附表",
                "监理人": "见申请人须知前附表",
                "代建人": "见申请人须知前附表",
                "资金来源": "见投标人须知前附表",
                "出资比例": "见投标人须知前附表",
                "资金落实情况": "见投标人须知前附表"
            },
            "招标范围": "见投标人须知前附表",
            "招标控制价": {
                "计税方法": "见投标人须知前附表",
                "总价": "见投标人须知前附表",
                "明细": "随招标文件发给所有下载招标文件的投标人",
                "编制依据": "《建设工程工程量清单计价规范》（GB50500-2013）和配套的工程量计算规范（GB50854-2013 ～GB50862-2013）、鄂建文[2013]39号《关于执行<房屋建筑与装饰工程工程量计算规范>（GB50584-2013）等 9项工程量计算规范有关问题的通知》、鄂建办[2018]27号《关于发布<湖北省房屋建筑与装饰工程消耗量定额及全费用基价表>等 8项定额的通知》、鄂建办〔2019〕93号《关于调整湖北省建设工程计价依据的通知》",
                "具体内容": "依据第五章工程量清单的说明"
            },
            "投标竞争下浮率": "5%",
            "分包": "不允许",
            "是否接受联合体投标": "见投标人须知前附表"
        },
        "关键时间/内容": {
            "投标文件递交截止日期": "见投标人须知前附表",
            "投标文件递交方式": "通过黄石市工程建设电子交易系统完成投标过程",
            "开标时间": "2022年 2月 25日 14时 00分",
            "开标地点": "本项目为不见面开标，所有投标人均应当准时在电子交易平台开标大厅在线参加开标",
            "投标人要求澄清招标文件的截止时间": "递交投标文件截止之日 16日前",
            "投标有效期": "自投标截止之日起 90日内有效",
            "评标结果公示媒介": [
                {
                    "名称": "大冶政府网（招投标板块）",
                    "网址": "http://36.133.17.9:8081/ztb/"
                },
                {
                    "名称": "黄石市公共资源交易网",
                    "网址": "https://www.hsztbzx.com/"
                }
            ]
        },
        "保证金相关": {
            "投标保证金": "不提交",
            "履约保证金": "不提交",
            "质量保证金": "未知",
            "退还投标保证金": "/"
        },
        "其他信息": {
            "重新招标、不再招标和终止招标": {
                "重新招标": "有下列情形之一的，招标人将重新招标：（1）投标截止时间止，投标人少于3个的；（2）经评标委员会评审后否决所有投标的；（3）中标人或所有定标候选人均未与招标人签订合同的；（4）法律、法规规定的其他情形。",
                "不再招标": "重新招标后投标人仍少于3个或者所有投标被否决的，属于必须审批或核准的工程建设项目，经原审批或核准部门批准后不再进行招标。",
                "终止招标": "因不可抗力等原因，招标人终止招标的，将通过“电子交易系统”及时发布公告，或者以书面形式通知被邀请的或者已经获取招标文件的潜在投标人。已经发出招标文件或者已经收取投标保证金的，招标人将及时退还所收取的招标文件的费用，以及所收取的投标保证金及银行同期存款利息。"
            },
            "投标费用承担": "投标人踏勘现场发生的费用自理。除招标人的原因外，投标人自行负责在踏勘现场中所发生的人员伤亡和财产损失。",
            "招标代理服务费": {
                "支付方": "由中标人支付",
                "支付标准": "参照国家发展计划委员会计价格[2002]1980号文工程类收费标准计取",
                "支付方式": "中标人按照约定的招标服务费通过电汇或转账等方式向招标代理机构支付",
                "支付时间": "中标人应当在中标通知书发出时，向招标代理机构一次性支付招标代理服务费"
            },
            "是否退还投标文件": "否",
            "偏离": "不允许",
            "投标预备会": "不召开",
            "踏勘现场": "不组织"
        }
    },
    "技术评分": {
        "工程概况": {
            "评分": "1分",
            "要求": "描述准确、清晰 1分；描述基本准确 0.9-0.1分；描述不准确 0分"
        },
        "施工部署": {
            "评分": "3分",
            "要求": "科学、合理、针对性强 3-2.8分；合理、可行 2.7-2.4分；欠合理，基本可行 2.3-2.1分；不可行，不能满足工程需要 0分"
        },
        "施工进度计划": {
            "评分": "3分",
            "要求": "科学、合理、针对性强 3-2.8分；合理、可行 2.7-2.4分；欠合理，基本可行 2.3-2.1分；不可行，不能满足招标文件要求 0分"
        },
        "施工准备与资源配置计划": {
            "评分": "2分",
            "要求": "内容完备，合理、针对性强 2-1.9分；内容完备，可行 1.8-1.6分；内容欠完备，基本可行 1.5-1.4分；不可行 0分"
        },
        "主要施工方案": {
            "评分": "5分",
            "要求": "科学、合理、针对性强 5-4.6分；合理、可行 4.5-4分；欠合理，基本可行 3.9-3.5分；不可行，不能满足工程需要 0分"
        },
        "施工现场平面布置": {
            "评分": "2分",
            "要求": "现场布置合理 2-1.9分；现场布置可行 1.8-1.6分；现场布置基本可行 1.5-1.4分；现场布置不可行 0分"
        },
        "主要施工管理计划": {
            "评分": "4分",
            "要求": "内容完备，合理、针对性强 4-3.7分；内容完备，可行 3.6-3.2分；内容欠完备，基本可行 3.1-2.8分；不可行 0分"
        }
    },
    "商务评分": {
        "商务评分": {
            "项目经理资格与业绩": {
                "职称": {
                    "评分": "1分",
                    "要求": "相关专业中级及以上职称（工程师） 1分；其他 0分"
                },
                "学历": {
                    "评分": "1分",
                    "要求": "专科及以上 1分；其他 0分"
                }
            },
            "技术负责人资格与业绩": {
                "职称": {
                    "评分": "1分",
                    "要求": "相关专业高级及以上职称（高级工程师） 1分；相关专业中级职称（工程师） 0.5分；其他 0分"
                },
                "学历": {
                    "评分": "1分",
                    "要求": "专科及以上 1分；其他 0分"
                },
                "从事专业工作年限": {
                    "评分": "1分",
                    "要求": "8年及以上 1分；5年-8年 0.8分；4年-2年 0.5分；不足 2年 0.2分"
                }
            },
            "其他主要人员": {
                "评分": "3分",
                "要求": "人员配备合理，满足需要 3分；人员配备基本合理,基本满足需要 2分；人员配备不合理 0分"
            },
            "类似项目业绩": {
                "评分": "2分",
                "要求": "近 5年有 1个类似项目业绩 2分【提供证明资料详见招标公告】"
            },
            "认证体系": {
                "评分": "3分",
                "要求": "具有有效的 ISO9001（GB/T19001）质量管理体系认证证书，ISO14001（GB/T24001）环境管理体系认证证，OHSAS45001（GB/T28001）职业健康安全管理体系认证证书。具备齐全得 3分，具备不全得 1分，不具备不得分。"
            },
            "近3年投标人获奖情况": {
                "评分": "1分",
                "要求": "近三年(2018年 12月至 2021年 12月）投标人承担过类似业绩中，每有 1个业绩获得市级及以上优质工程荣誉或安全文明施工现场奖的，得 0.5分，最多得 1分。需提供荣誉证书或获奖证明文件，以有效证明的落款时间为准。注：以证书颁发日期为准，同一项目以最高奖项计算不重复得分。"
            },
            "信用": {
                "评分": "2分",
                "要求": "未受到建设行政管理部门做出的行政处罚的得 2分；有一项扣 0.1分，扣完为止。（查询结果以评标委员会评标当天在“信用中国”查询投标人有行政处罚记录信息且在公示期内的予以记分 0.1分/次），同一事由被不同行政管理部门处罚的不重复扣分，分值扣完为止。"
            }
        },
        "投标报价评审": {
            "有效投标报价确定": {
                "评分": "合格/不合格",
                "要求": "低于招标控制价"
            },
            "有效投标报价的排序": {
                "评分": "合格/不合格",
                "要求": "有效投标报价由低到高排序"
            },
            "投标文件基础数据分析和整理": {
                "评分": "合格/不合格",
                "要求": "清标报告"
            },
            "投标报价成本分析": {
                "评分": "合格/不合格",
                "要求": "清标报告"
            },
            "投标竞争下浮率": {
                "评分": "合格/不合格",
                "要求": "取值：5%（投标竞争下浮率的值为整数，具体由招标人结合项目特点和需要在招标文件中明确）"
            },
            "投标报价最终排序": {
                "评分": "合格/不合格",
                "要求": "低于招标控制价且超过投标竞争下浮率"
            },
            "投标报价不合格的情形": {
                "评分": "合格/不合格",
                "要求": "仅限于投标报价高于最高投标限价或低于成本的情形。投标报价文件评定为“合格”的，评标委员会应列出报价组成不合理、不平衡报价、签订合同前应注意和澄清的事项。"
            }
        }
    },
    "资格审查": {
        "形式评审标准": {
            "投标文件": "投标文件能正常打开",
            "投标人名称": "与营业执照、资质证书、安全生产许可证一致",
            "投标文件签字盖章": "电子投标文件制作的具体方法详见“电子投标文件制作软件”中的帮助文档。",
            "投标文件格式、内容": "符合第八章“投标文件格式”的要求，实质性内容齐全、关键 字迹清晰可辨",
            "联合体投标人（如有）": "提交联合体协议书，并明确联合体牵头人",
            "报价唯一": "只能有一个有效报价（指投标函中的大写报价）",
            "多标段投标": "投标人可同时对本次招标标段中的/个标段投标。招标人按下列原则选择中标人：<br>□招标人按标段择优选择中标人。<br>□投标人最多只允许中标个标段。",
            "“技术暗标”": "任何情况下，施工组织设计中不得出现投标人的名称和其它可识别投标人身份的字符、徽标、人员名称等。"
        },
        "响应性评审标准": {
            "投标内容": "完成本项目施工图纸范围内全部内容，具体以工程量清单为准。关于招标范围的详细说明见第七章“技术标准和要求”。",
            "工期": "计划工期：300日历天<br>计划开工日期：2022年03月<br>计划竣工日期：2022年12月<br>除上述总工期外，发包人还要求以下区段工期：<br>\t/<br>有关工期的详细要求见第七章“技术标准和要求”。",
            "工程质量": "质量目标：合格<br>关于质量要求的详细说明见第七章“技术标准和要求”。",
            "投标有效期": "自投标截止之日起90日内有效",
            "投标保证金": "不提交",
            "权利义务": "投标函附录中的相关承诺符合或优于第四章“合同条款及格式”的相关规定",
            "技术标准和要求": "符合第七章“技术标准和要求”规定",
            "分包计划": "不允许",
            "算术错误修正": "1)投标人接受算术错误修正后的报价 2)修正后的报价与投标报价相比偏差率不超过±1%",
            "投标价格": "1）投标函中的大写报价与已标价工程量清单中的投标总价一致 2）投标函中的大写报价不大于本标段招标控制价总价 3）算术错误修正后的投标总报价不大于本标段招标控制价总价 4）投标报价不低于其成本",
            "已标价工程量清单": "1）已标价工程量清单项目编码顺序与第五章“工程量清单”给 出的项目编码顺序一致； 2）已标价工程量清单符合第五章“工程量清单”给出的项目编 码、项目名称、项目特征、计量单位和工程量； 3）暂列金额符合第五章“工程量清单”列出的金额； 4）专业工程暂估价符合第五章“工程量清单”列出的金额； 5）材料（工程设备）暂估价符合第五章“工程量清单”列出的 单价并计入综合单价； 6）安全文明施工费、规费和税金等不可竞争费用，按照规定 的标准计取； 7）计税方法符合招标文件的约定； 8）已标价工程量清单项目未填报的项目不超过三项，或不超 过三项未填报的项目的费用合计（按招标控制价相应项目的费 用合计计算）不超过其投标总报价（修正后的投标总报价，如 有）的±1%"
        },
        "资格评审": {
            "资质等级": "具备住房城乡建设主管部门颁发的有效的市政公用工程施工总承包贰级及以上资质证书，具备有效的安全生产许可证，具备市场监督管理部门核发的有效企业法人营业执照。",
            "财务状况": "1.投标人近三年平均利润大于 0元。2.投标人须提供近三年（2018、2019、2020年）的财务审计报告（至少包括资产负债表、现金流量表、利润表）。",
            "类似项目业绩": "近 5年（投标截止日前 5年）至少有一项市政公用工程的施工业绩（提供中标通知书或施工合同或工程接收证书或工程竣工验收证明，以竣工时间为准）",
            "信誉": "1.没有被依法暂停或取消投标资格，或没有处于财产被接管、冻结；2.没有被责令停产停业、暂扣或者吊销许可证、暂扣或者吊销执照；3.没有进入清算程序，或被宣告破产，或其他丧失履约能力的情形；4.在最近三年内没有骗取中标和严重违约及发生重大工程质量问题；5.在“国家企业信用信息公示系统”（ www.gsxt.gov.cn）没有被列入严重违法失信企业名单；6.在“信用中国”网站（www.creditchina.gov.cn）没有被列入失信被执行人名单；7.在近三年内投标人或其法定代表人、拟委任的项目经理没有行贿犯罪行为；8.不存在法律法规或投标人须知前附表第 1.4.3（19）目规定的不得存在的其他情形。",
            "其他要求": "1.项目管理机构主要人员不得本项目中相互兼职。2.均应当在投标人处注册执业或岗位登记，且社会保险缴费单位应是投标人。",
            "项目经理": {
                "数量": "1人",
                "资格要求": "具备市政公用工程专业贰级及以上注册建造师执业资格和有效的安全生产考核合格证书（B证），且未担任其他在施建设工程项目的项目经理。",
                "需要提交的证明材料": "注册建造师执业资格证书、安全生产考核合格证书（B证）",
                "在岗要求": "未担任其他在施建设工程项目的项目经理"
            },
            "项目技术负责人": {
                "数量": "1人",
                "资格要求": "具备市政公用工程相关专业中级及以上职称或市政公用工程专业贰级及以上注册建造师执业资格，5年以上主管施工技术经验。",
                "需要提交的证明材料": "职称证书或注册建造师执业资格证书",
                "在岗要求": "无"
            },
            "施工管理": {
                "数量": "1人",
                "资格要求": "持有施工员岗位培训考核合格证书。",
                "需要提交的证明材料": "施工员岗位培训考核合格证书",
                "在岗要求": "无"
            },
            "质量管理": {
                "数量": "1人",
                "资格要求": "持有质量员岗位培训考核合格证书。",
                "需要提交的证明材料": "质量员岗位培训考核合格证书",
                "在岗要求": "无"
            },
            "安全管理": {
                "数量": "1人",
                "资格要求": "持有有效的安全生产考核合格证书（C证）。",
                "需要提交的证明材料": "安全生产考核合格证书（C证）",
                "在岗要求": "无"
            },
            "项目管理机构主要人员": {
                "数量": "1.项目管理机构主要人员不得本项目中相互兼职。2.均应当在投标人处注册执业或岗位登记，且社会保险缴费单位应是投标人。",
                "资格要求": "无",
                "需要提交的证明材料": "注册执业证书或岗位登记证明、社保证明",
                "在岗要求": "不得本项目中相互兼职，均应当在投标人处注册执业或岗位登记，且社会保险缴费单位应是投标人"
            },
            "联合体投标要求（如有）": {
                "联合体协议书": "（3）联合体协议书；",
                "联合体投标的规定": "3.1.2投标人须知前附表规定不接受联合体投标的，或投标人没有组成联合体的，投标文件不包括本章第 3.1.1（3）目所指的联合体协议书。",
                "联合体投标的保证金": "联合体投标的，其投标保证金由牵头人递交，并应符合投标人须知前附表的规定。",
                "联合体投标的资格审查资料": "3.5.2投标人须知前附表第 1.4.2项规定接受联合体投标的，本章第 3.5.1项规定的表格和资料应包括联合体各方相关情况。",
                "联合体投标的其他要求": "1.4.2投标人须知前附表规定接受联合体投标的，除应符合本章第 1.4.1项和投标人须知前附表的要求外，还应遵守以下规定：（1）联合体各方应按招标文件提供的格式签订联合体协议书，明确联合体牵头人和各方权利义务；（2）由同一专业的单位组成的联合体，按照资质等级较低的单位确定资质等级；（3）联合体各方不得再以自己名义单独或参加其他联合体在同二标段中投标。"
            },
            "营业执照": "具备有效的营业执照",
            "安全生产许可证": "具备有效的安全生产许可证"
        }
    }
}
    includes = ["基础信息", "资格审查", "商务评分", "技术评分", "无效标与废标项", "投标文件要求", "开评定标流程"]
    res1,res2,res3=outer_post_processing(combined_data,includes)
    print(json.dumps(res2,ensure_ascii=False,indent=4))


